// Copyright (C) 2009 Playedit.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "config.h"

#include "basic_dialogs.h"

#include <glibmm/i18n.h>
#include <glibmm/miscutils.h>
#include <giomm/appinfo.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/aboutdialog.h>
#include <gtkmm/filechooserdialog.h>
#include <gtkmm/stock.h>
#include <iostream>

// Run a simple confirmation dialog.  Returns true for ok.
bool ConfirmationDialog(const Glib::ustring& title,
		const Glib::ustring& label_txt)
{
	Gtk::MessageDialog message_dialog(title, false,
			Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK_CANCEL, true);
	message_dialog.set_title(title);
	message_dialog.set_secondary_text(label_txt);
	return message_dialog.run() == Gtk::RESPONSE_OK;
}

// A simple dialog used to get text input.
EntryDialog::EntryDialog(const Glib::ustring& title, const Glib::ustring& frame_txt,
		const Glib::ustring& entry_txt)
	: Gtk::Dialog(title, true),
	m_Frame(frame_txt)
{
	add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	Gtk::Button* ok_button = add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
	ok_button->set_can_default();
	ok_button->grab_default();

	m_Entry.set_text(entry_txt);
	m_Entry.set_activates_default();

	m_Frame.add(m_Entry);
	get_content_area()->pack_start(m_Frame, false, false);
	show_all();
}

Glib::ustring EntryDialog::GetEntry()
{
	return m_Entry.get_text();
}

// Run an entry dialog.  Return the text that is entered.
Glib::ustring EntryDialog::RunEntryDialog(const Glib::ustring& title, const Glib::ustring& frame_txt,
		const Glib::ustring& entry_txt)
{
	EntryDialog dialog(title, frame_txt, entry_txt);

	if (dialog.run() == Gtk::RESPONSE_OK)
		return dialog.GetEntry();
	else
		return "";
}

static void UpdatePreview(Gtk::FileChooserDialog& dialog, Gtk::Image& preview)
{
	Glib::ustring filename = dialog.get_preview_filename();
	if (filename.size() &&
			Gio::File::create_for_path(filename)->query_info()->get_file_type() != Gio::FILE_TYPE_DIRECTORY)
	{
		preview.set(Gdk::Pixbuf::create_from_file(filename, 120, 120));
		dialog.set_preview_widget_active(true);
	}
	else
		dialog.set_preview_widget_active(false);
}

// Run a file chooser dialog.  Returns the selected filename.
Glib::ustring ImageChooserDialog(const Glib::ustring& title)
{
	Gtk::FileChooserDialog dialog(title);

	dialog.set_local_only();
	dialog.set_current_folder(Glib::get_user_special_dir(G_USER_DIRECTORY_PICTURES));
	dialog.add_shortcut_folder(Glib::get_user_special_dir(G_USER_DIRECTORY_PICTURES));
	dialog.add_shortcut_folder(Glib::get_user_special_dir(G_USER_DIRECTORY_MUSIC));

	Glib::RefPtr<Gtk::FileFilter> filter = Gtk::FileFilter::create();
	filter->add_pixbuf_formats();
	dialog.set_filter(filter);

	Gtk::Image preview;
	dialog.set_preview_widget(preview);
	dialog.signal_update_preview().connect(
			sigc::bind<Gtk::FileChooserDialog&, Gtk::Image&>(&UpdatePreview, dialog, preview));

	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);

	if (dialog.run() == Gtk::RESPONSE_ACCEPT)
		return dialog.get_filename();
	else
		return "";
}

// Show the about dialog.
void AboutDialog()
{
	Gtk::AboutDialog dialog;
	dialog.set_modal();

	dialog.set_program_name(_("PlayEdit"));
	dialog.set_version(VERSION);
	dialog.set_comments(_("PlayEdit is a graphical interface "
				"for managing playlists "
				"on MTP devices."));
	dialog.set_copyright(_("Copyright \xC2\xA9 2009 "
				"The PlayEdit author"));
	dialog.set_license(
			_("PlayEdit is free software: you can redistribute it and/or modify\n"
				"it under the terms of the GNU General Public License as published by\n"
				"the Free Software Foundation, either version 3 of the License, or\n"
				"(at your option) any later version.\n"
				"\n"
				"This program is distributed in the hope that it will be useful,\n"
				"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
				"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
				"GNU General Public License for more details.\n"
				"\n"
				"You should have received a copy of the GNU General Public License\n"
				"along with this program.  If not, see http://www.gnu.org/licenses/."));
	dialog.set_authors(std::vector<Glib::ustring>(1, "Glenn Rice"));
	dialog.set_website("https://sourceforge.net/projects/playedit/");
	dialog.set_website_label("PlayEdit on SourceForge");

	dialog.run();
}

// Show the gnome help for playedit.
void LaunchHelp()
{
	try
	{
		Gio::AppInfo::launch_default_for_uri("ghelp:playedit");
	}
	catch (const Glib::Error& ex)
	{
		std::cerr << _("Failed to launch help: ") <<  ex.what();
	}
}
