// Copyright (C) 2009 Playedit.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "config.h"

#include "playlist.h"
#include "music.h"
#include "basic_dialogs.h"

#include <glibmm/i18n.h>
#include <gtkmm/main.h>
#include <gtkmm/window.h>
#include <gtkmm/stock.h>
#include <gtkmm/menubar.h>
#include <iostream>

static void StartEditingCell(PlaylistView*& playlist_treeview)
{
	playlist_treeview->SetPlaylistNameEditable();
}

static void NewPlaylist(PlaylistView*& playlist_treeview)
{
	playlist_treeview->NewPlaylist();
}

static void DeletePlaylists(PlaylistView*& playlist_treeview)
{
	playlist_treeview->DeletePlaylists();
}

static void RemoveTracksFromPlaylist(PlaylistView*& playlist_treeview)
{
	playlist_treeview->RemoveTracksFromPlaylist();
}

static void EditAlbumMetadata(MusicView*& music_treeview)
{
	music_treeview->EditAlbumMetadata();
}

static void AddAlbumArt(MusicView*& music_treeview)
{
	music_treeview->AddAlbumArt();
}

static void EditTrackMetadata(MusicView*& music_treeview)
{
	music_treeview->EditTrackMetadata();
}

static void DeleteTracksFromDevice(MusicView*& music_treeview)
{
	music_treeview->DeleteTracksFromDevice();
}

static void DeleteAlbumsFromDevice(MusicView*& music_treeview)
{
	music_treeview->DeleteAlbumsFromDevice();
}

static void SortByName(MusicView*& music_treeview)
{
	music_treeview->SetSortByName();
}

static void SortByArtist(MusicView*& music_treeview)
{
	music_treeview->SetSortByArtist();
}

namespace Playedit
{

void App::CreateUIManager()
{
	Glib::RefPtr<Gtk::ActionGroup> main_action_group = Gtk::ActionGroup::create("MainActions");

	// Help menu actions
	main_action_group->add(Gtk::Action::create("HelpMenuAction", _("_Help")));

	main_action_group->add(Gtk::Action::create("HelpAction", Gtk::Stock::HELP,
				_("_Help"), _("Help")), &LaunchHelp);

	main_action_group->add(Gtk::Action::create("AboutAction", Gtk::Stock::ABOUT,
				_("_About"), _("About MTP Playlist Editor")), &AboutDialog);

	// File menu actions
	main_action_group->add(Gtk::Action::create("FileMenuAction", _("_File")));

	main_action_group->add(Gtk::Action::create("QuitAction", Gtk::Stock::QUIT, _("_Quit"), _("Quit")),
			Gtk::AccelKey("<control>Q"), sigc::mem_fun(*this, &App::QuitPlayedit));

	// Device menu actions
	m_DeviceInteractionGroup = Gtk::ActionGroup::create("DeviceActions");

	m_DeviceInteractionGroup->add(Gtk::Action::create("DeviceMenuAction", _("_Device")));

	m_DeviceInteractionGroup->add(Gtk::Action::create("DisconnectDeviceAction", Gtk::Stock::DISCONNECT,
				_("_Disconnect Device"), _("Disconnect device")), Gtk::AccelKey("<control>D"),
			sigc::mem_fun(*this, &Playedit::App::DisconnectDevice));

	Glib::RefPtr<Gtk::Action> rename_action = Gtk::Action::create("RenameDeviceAction",
			_("_Rename Device"), _("Rename device"));
	rename_action->set_icon_name("RenameDevice");
	m_DeviceInteractionGroup->add(rename_action, Gtk::AccelKey("<control>R"),
			sigc::mem_fun(*this, &Playedit::App::RenameDevice));

	// Playlist actions
	m_DeviceActionGroup = Gtk::ActionGroup::create("PlaylistActions");

	m_DeviceActionGroup->add(Gtk::Action::create("CreatePlaylistAction",
				Gtk::Stock::NEW, _("_Create Playlist"), _("Create playlist (Ctrl+Ins)")),
			Gtk::AccelKey("<control>Insert"),
			sigc::bind<PlaylistView*&>(&NewPlaylist, m_PlaylistView));

	m_DeviceActionGroup->add(Gtk::Action::create("RenamePlaylistAction",
				Gtk::Stock::EDIT, _("_Rename Playlist"), _("Rename playlist (F2)")),
			Gtk::AccelKey("F2"),
			sigc::bind<PlaylistView*&>(&StartEditingCell, m_PlaylistView));

	m_DeviceActionGroup->add(Gtk::Action::create("DeletePlaylistsAction",
				Gtk::Stock::DELETE, _("_Delete Playlists"), _("Delete playlists (Ctrl+Del)")),
			Gtk::AccelKey("<control>Delete"),
			sigc::bind<PlaylistView*&>(&DeletePlaylists, m_PlaylistView));

	m_DeviceActionGroup->add(Gtk::Action::create("RemoveTracksAction",
				Gtk::Stock::DELETE, _("_Remove Tracks"),
				_("Remove selected tracks from playlist (Ctrl+Del)")),
			Gtk::AccelKey("<control>Delete"),
			sigc::bind<PlaylistView*&>(&RemoveTracksFromPlaylist, m_PlaylistView));

	// Music actions
	m_DeviceActionGroup->add(Gtk::Action::create("EditAlbumMetadataAction",
				Gtk::Stock::EDIT, _("_Edit Album Metadata"),
				_("Edit album metadata (F3)")),
			Gtk::AccelKey("F3"),
			sigc::bind<MusicView*&>(&EditAlbumMetadata, m_MusicView));

	m_DeviceActionGroup->add(Gtk::Action::create("AddAlbumArtAction",
				Gtk::Stock::ORIENTATION_PORTRAIT, _("_Add/Change Album Art"),
				_("Add or change art for album on device (F4)")),
			Gtk::AccelKey("F4"),
			sigc::bind<MusicView*&>(&AddAlbumArt, m_MusicView));

	m_DeviceActionGroup->add(Gtk::Action::create("EditTrackMetadataAction",
				Gtk::Stock::EDIT, _("_Edit Track Metadata"),
				_("Edit album metadata (F3)")),
			Gtk::AccelKey("F3"),
			sigc::bind<MusicView*&>(&EditTrackMetadata, m_MusicView));

	m_DeviceActionGroup->add(Gtk::Action::create("DeleteTracksAction",
				Gtk::Stock::DELETE, _("_Delete Tracks"), _("Delete selected tracks from device (Alt+Del)")),
			Gtk::AccelKey("<alt>Delete"),
			sigc::bind<MusicView*&>(&DeleteTracksFromDevice, m_MusicView));

	m_DeviceActionGroup->add(Gtk::Action::create("DeleteAlbumsAction",
				Gtk::Stock::DELETE, _("_Delete Albums"), _("Delete selected albums from device (Alt+Del)")),
			Gtk::AccelKey("<alt>Delete"),
			sigc::bind<MusicView*&>(&DeleteAlbumsFromDevice, m_MusicView));

	// Music sorting actions
	Glib::RefPtr<Gtk::ActionGroup> music_sort_action_group = Gtk::ActionGroup::create("MusicSortActions");

	music_sort_action_group->add(Gtk::Action::create("SortByNameAction", _("Sort by _Name"), _("Sort by name")),
			sigc::bind<MusicView*&>(&SortByName, m_MusicView));

	music_sort_action_group->add(Gtk::Action::create("SortByArtistAction", _("Sort by _Artist"), _("Sort by artist")),
			sigc::bind<MusicView*&>(&SortByArtist, m_MusicView));

	const Glib::ustring ui_description =
		"<ui>"
		"  <menubar name='MainMenu'>"
		"    <menu name='File' action='FileMenuAction'>"
		"      <menuitem name='Quit' action='QuitAction'/>"
		"      <placeholder name='FileMenuAdditions'/>"
		"    </menu>"
		"    <menu name='Device' action='DeviceMenuAction'>"
		"      <menuitem name='DisconnectDevice' action='DisconnectDeviceAction'/>"
		"      <menuitem name='RenameDevice' action='RenameDeviceAction'/>"
		"      <placeholder name='DeviceMenuAdditions'/>"
		"    </menu>"
		"    <menu name='Help' action='HelpMenuAction'>"
		"      <menuitem name='Help' action='HelpAction'/>"
		"      <menuitem name='About' action='AboutAction'/>"
		"      <placeholder name='HelpMenuAdditions'/>"
		"    </menu>"
		"  </menubar>"
		"  <toolbar name='MainToolbar' action='MainToolbarAction'>"
		"    <placeholder name='ToolItems'>"
		"      <separator/>"
		"      <toolitem name='Quit' action='QuitAction'/>"
		"      <separator/>"
		"      <toolitem name='DisconnectDevice' action='DisconnectDeviceAction'/>"
		"      <toolitem name='RenameDevice' action='RenameDeviceAction'/>"
		"      <separator/>"
		"      <toolitem name='Help' action='HelpAction'/>"
		"      <toolitem name='About' action='AboutAction'/>"
		"    </placeholder>"
		"  </toolbar>"
		"  <popup name='PlaylistPopup'>"
		"    <menuitem name='CreatePlaylist' action='CreatePlaylistAction'/>"
		"    <menuitem name='RenamePlaylist' action='RenamePlaylistAction'/>"
		"    <menuitem name='DeletePlaylists' action='DeletePlaylistsAction'/>"
		"    <menuitem name='RemoveTracks' action='RemoveTracksAction'/>"
		"  </popup>"
		"  <toolbar name='PlaylistToolbar'>"
		"    <toolitem name='CreatePlaylist' action='CreatePlaylistAction'/>"
		"    <toolitem name='RenamePlaylist' action='RenamePlaylistAction'/>"
		"    <toolitem name='DeletePlaylists' action='DeletePlaylistsAction'/>"
		"    <toolitem name='RemoveTracks' action='RemoveTracksAction'/>"
		"  </toolbar>"
		"  <popup name='MusicPopup'>"
		"    <menuitem name='EditAlbumMetadata' action='EditAlbumMetadataAction'/>"
		"    <menuitem name='AddAlbumArt' action='AddAlbumArtAction'/>"
		"    <menuitem name='EditTrackMetadata' action='EditTrackMetadataAction'/>"
		"    <menuitem name='DeleteTracks' action='DeleteTracksAction'/>"
		"    <menuitem name='DeleteAlbums' action='DeleteAlbumsAction'/>"
		"  </popup>"
		"  <toolbar name='MusicToolbar'>"
		"    <toolitem name='EditAlbumMetadata' action='EditAlbumMetadataAction'/>"
		"    <toolitem name='AddAlbumArt' action='AddAlbumArtAction'/>"
		"    <toolitem name='EditTrackMetadata' action='EditTrackMetadataAction'/>"
		"    <toolitem name='DeleteTracks' action='DeleteTracksAction'/>"
		"    <toolitem name='DeleteAlbums' action='DeleteAlbumsAction'/>"
		"  </toolbar>"
		"  <popup name='MusicSortPopup'>"
		"    <menuitem name='SortByName' action='SortByNameAction'/>"
		"    <menuitem name='SortByArtist' action='SortByArtistAction'/>"
		"  </popup>"
		"</ui>";

	m_UIManager = Gtk::UIManager::create();

	m_UIManager->insert_action_group(main_action_group);
	m_UIManager->insert_action_group(m_DeviceInteractionGroup);
	m_UIManager->insert_action_group(m_DeviceActionGroup);
	m_UIManager->insert_action_group(music_sort_action_group);

	try
	{
		m_UIManager->add_ui_from_string(ui_description);
	}
	catch(const Glib::Error& ex)
	{
		std::cerr << "Building UI failed: " <<  ex.what() << std::endl;
	}
}

// Set up the music toolbar
Gtk::Toolbar* App::GetMusicToolbar()
{
	Gtk::Toolbar* music_toolbar =
		(Gtk::Toolbar*)m_UIManager->get_widget("/MusicToolbar");
	music_toolbar->set_toolbar_style(Gtk::TOOLBAR_ICONS);
	music_toolbar->set_icon_size(Gtk::ICON_SIZE_SMALL_TOOLBAR);
	((Gtk::Orientable*)music_toolbar)->set_orientation(Gtk::ORIENTATION_VERTICAL);
	music_toolbar->set_size_request(34, -1);

	return music_toolbar;
}

// Set up the playlist toolbar
Gtk::Toolbar* App::GetPlaylistToolbar()
{
	Gtk::Toolbar* playlist_toolbar =
		(Gtk::Toolbar*)m_UIManager->get_widget("/PlaylistToolbar");
	playlist_toolbar->set_toolbar_style(Gtk::TOOLBAR_ICONS);
	playlist_toolbar->set_icon_size(Gtk::ICON_SIZE_SMALL_TOOLBAR);
	((Gtk::Orientable*)playlist_toolbar)->set_orientation(Gtk::ORIENTATION_VERTICAL);

	return playlist_toolbar;
}

}
