// Copyright (C) 2009 Playedit.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "config.h"

#include "metadata_editor.h"

#include <glibmm/i18n.h>
#include <glibmm/stringutils.h>
#include <gtkmm/stock.h>
#include <iomanip>

BoldLabel::BoldLabel(const Glib::ustring &text, Gtk::Entry &mnemonic_widget)
	: Gtk::Label("<b>" + text + "</b>", true)
{
	set_use_markup();
	set_alignment(1.0f, 0.5f);
	set_mnemonic_widget(mnemonic_widget);
}

NumericSpinButton::NumericSpinButton(int initial_value, int max_value)
{
	m_Adjustment = Gtk::Adjustment::create(initial_value, 0.0, max_value);
	set_adjustment(m_Adjustment);
	set_wrap();
	set_numeric();
	set_alignment(Gtk::ALIGN_END);
}

AlbumMetadataEditor::AlbumMetadataEditor(const Glib::RefPtr<album_t>& album, const Glib::ustring& message)
	: Gtk::Dialog(_("Album Metadata Editor"), true),
	m_MessageLabel(message),
	m_NameLabel(_("_Name:"), m_NameEntry),
	m_ArtistLabel(_("_Artist:"), m_ArtistEntry),
	m_GenreLabel(_("_Genre:"), m_GenreEntry),
	m_Album(album)
{
	// Message
	if (message.size())
	{
		m_MessageLabel.set_line_wrap();
		m_Frame.add(m_MessageLabel);
		get_content_area()->pack_start(m_Frame, true, true);
	}

	m_Grid.set_column_spacing(16);
	m_Grid.set_row_spacing(8);

	// Album name
	m_NameEntry.set_text(album->RawName());
	m_NameEntry.set_hexpand();
	m_Grid.attach(m_NameLabel, 0, 0, 1, 1);
	m_Grid.attach(m_NameEntry, 1, 0, 1, 1);

	// Album artist
	m_ArtistEntry.set_text(album->RawArtist());
	m_ArtistEntry.set_hexpand();
	m_Grid.attach(m_ArtistLabel, 0, 1, 1, 1);
	m_Grid.attach(m_ArtistEntry, 1, 1, 1, 1);

	// Album genre
	m_GenreEntry.set_text(album->Genre());
	m_GenreEntry.set_hexpand();
	m_Grid.attach(m_GenreLabel, 0, 2, 1, 1);
	m_Grid.attach(m_GenreEntry, 1, 2, 1, 1);

	// Dialog buttons
	add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

	get_content_area()->pack_start(m_Grid, true, true);

	set_default_size(400, -1);

	show_all();
}

void AlbumMetadataEditor::on_response(int response_id)
{
	if (response_id == Gtk::RESPONSE_OK)
	{
		m_Album->SetName(m_NameEntry.get_text());
		m_Album->SetArtist(m_ArtistEntry.get_text());
		m_Album->SetGenre(m_GenreEntry.get_text());
	}
}

TrackMetadataEditor::TrackMetadataEditor(const Glib::RefPtr<track_t>& track)
	: Gtk::Dialog(_("Track Metadata Editor"), true),
	m_TrackDateSpin(track->Date().size() ? Glib::Ascii::strtod(track->Date()) : 0, 3000),
	m_TrackNumSpin(track->TrackNumber(), 100),
	m_DurationMinSpin(track->Duration() / 60000, 1440),
	m_DurationSecSpin((track->Duration() % 60000) / 1000, 60),
	m_DurationMsSpin(track->Duration() % 1000, 1000),
	m_NameLabel(_("_Title:"), m_NameEntry),
	m_AlbumLabel(_("A_lbum:"), m_AlbumEntry),
	m_ArtistLabel(_("_Artist:"), m_ArtistEntry),
	m_GenreLabel(_("_Genre:"), m_GenreEntry),
	m_TrackDateLabel(_("_Date:"), m_TrackDateSpin),
	m_TrackNumLabel(_("Track _Number:"), m_TrackNumSpin),
	m_DurationLabel(_("D_uration:"), m_DurationMinSpin),
	m_DurationMinLabel(_("min ")),
	m_DurationSecLabel(_("sec ")),
	m_DurationMsLabel(_("ms")),
	m_Track(track)
{
	m_Grid.set_column_spacing(16);
	m_Grid.set_row_spacing(8);

	// Track name
	m_NameEntry.set_text(track->Name());
	m_NameEntry.set_hexpand();
	m_Grid.attach(m_NameLabel, 0, 0, 1, 1);
	m_Grid.attach(m_NameEntry, 1, 0, 1, 1);

	// Track album
	m_AlbumEntry.set_text(track->RawAlbum());
	m_AlbumEntry.set_hexpand();
	m_Grid.attach(m_AlbumLabel, 0, 1, 1, 1);
	m_Grid.attach(m_AlbumEntry, 1, 1, 1, 1);

	// Track artist
	m_ArtistEntry.set_text(track->RawArtist());
	m_ArtistEntry.set_hexpand();
	m_Grid.attach(m_ArtistLabel, 0, 2, 1, 1);
	m_Grid.attach(m_ArtistEntry, 1, 2, 1, 1);

	// Track genre
	m_GenreEntry.set_text(track->Genre());
	m_GenreEntry.set_hexpand();
	m_Grid.attach(m_GenreLabel, 0, 3, 1, 1);
	m_Grid.attach(m_GenreEntry, 1, 3, 1, 1);

	// Track date
	m_TrackDateSpin.set_hexpand();
	m_Grid.attach(m_TrackDateLabel, 0, 4, 1, 1);
	m_Grid.attach(m_TrackDateSpin, 1, 4, 1, 1);

	// Track number
	m_TrackNumSpin.set_hexpand();
	m_Grid.attach(m_TrackNumLabel, 0, 5, 1, 1);
	m_Grid.attach(m_TrackNumSpin, 1, 5, 1, 1);

	// Track duration
	m_DurationBox.pack_start(m_DurationMinSpin, Gtk::PACK_SHRINK);
	m_DurationBox.pack_start(m_DurationMinLabel, Gtk::PACK_SHRINK);
	m_DurationBox.pack_start(m_DurationSecSpin, Gtk::PACK_SHRINK);
	m_DurationBox.pack_start(m_DurationSecLabel, Gtk::PACK_SHRINK);
	m_DurationBox.pack_start(m_DurationMsSpin, Gtk::PACK_EXPAND_WIDGET);
	m_DurationBox.pack_start(m_DurationMsLabel, Gtk::PACK_SHRINK);
	m_DurationBox.set_hexpand();
	m_Grid.attach(m_DurationLabel, 0, 6, 1, 1);
	m_Grid.attach(m_DurationBox, 1, 6, 1, 1);

	// Dialog buttons
	add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

	get_content_area()->pack_start(m_Grid, true, true);

	set_default_size(400, -1);

	show_all();
}

void TrackMetadataEditor::on_response(int response_id)
{
	if (response_id == Gtk::RESPONSE_OK)
	{
		m_Track->SetTitle(m_NameEntry.get_text());
		m_Track->SetAlbum(m_AlbumEntry.get_text());
		m_Track->SetArtist(m_ArtistEntry.get_text());
		m_Track->SetGenre(m_GenreEntry.get_text());
		if (m_TrackDateSpin.get_value_as_int())
			m_Track->SetDate(Glib::ustring::format(std::setfill(L'0'), std::setw(4),
						m_TrackDateSpin.get_value_as_int()));
		m_Track->SetTrackNumber(m_TrackNumSpin.get_value_as_int());
		m_Track->SetDuration(m_DurationMinSpin.get_value_as_int() * 60000 +
				m_DurationSecSpin.get_value_as_int() * 1000 + m_DurationMsSpin.get_value_as_int());
	}
}
