// Copyright (C) 2009 Playedit.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "config.h"

#include "mtp_device.h"

#include <glibmm/i18n.h>
#include <giomm/memoryinputstream.h>
#include <libmtp.h>
#include <iostream>

MTPDevice::MTPDevice(int (*callback)(const uint64_t, const uint64_t, const void*),
		void* callback_data)
	: m_Device(NULL),
	m_DevNum(0),
	m_ProgressCallback(callback),
	m_CallbackData(callback_data),
	m_IsReady(false)
{
	LIBMTP_Init();
}

MTPDevice::~MTPDevice()
{
	Disconnect();
}

void MTPDevice::Connect(int devnum)
{
	if (m_Device) return;

	m_IsReady = false;
	int num_raw_devices;
	LIBMTP_raw_device_t *raw_devices = NULL;
	if (LIBMTP_Detect_Raw_Devices(&raw_devices, &num_raw_devices) == 0)
	{
		int i;
		for (i = 0; i < num_raw_devices; i++)
		{
			if (devnum == raw_devices[i].devnum)
			{
				m_DevNum = raw_devices[i].devnum;
				break;
			}
		}
		if (m_DevNum)
			m_Device = LIBMTP_Open_Raw_Device(&raw_devices[i]);

		free(raw_devices);
	}
}

void MTPDevice::SetReady(const bool ready)
{
	m_IsReady = ready;
}

void MTPDevice::Disconnect()
{
	m_IsReady = false;
	if (m_Device) LIBMTP_Release_Device(m_Device);
	m_Device = NULL;
	m_DevNum = 0;
}

Glib::ustring MTPDevice::DeviceName()
{
	char* dev_name = LIBMTP_Get_Friendlyname(m_Device);
	Glib::ustring ret_dev_name = dev_name ? dev_name : "";
	g_free(dev_name);
	return ret_dev_name;
}

void MTPDevice::SetDeviceName(const Glib::ustring& name)
{
	LIBMTP_Set_Friendlyname(m_Device, name.c_str());
}

void MTPDevice::DumpErrorStack(const Glib::ustring& message)
{
	g_message("%s", message.c_str());
	LIBMTP_Dump_Errorstack(m_Device);
	LIBMTP_Clear_Errorstack(m_Device);
}

// Get the album art for the album from the device
void MTPDevice::GetAlbumArt(const Glib::RefPtr<album_t>& album)
{
	if (m_Device == NULL || album->album == NULL || album->album->album_id == 0)
		return;

	albumart_t thumb;
	if (LIBMTP_Get_Representative_Sample(m_Device, album->album->album_id, thumb.data))
		DumpErrorStack(_("Failed to retrieve thumbnail from device."));
	else if (thumb.data->size)
	{
		Glib::RefPtr<Gio::MemoryInputStream> image_stream = Gio::MemoryInputStream::create();
		image_stream->add_data(thumb.data->data, thumb.data->size);
		album->art = Gdk::Pixbuf::create_from_stream_at_scale(image_stream, -1, 120, true);
	}
}

// Updates a playlist on the device.  Returns true if successful.
bool MTPDevice::UpdatePlaylist(const Glib::RefPtr<playlist_t>& playlist)
{
	if (LIBMTP_Update_Playlist(m_Device, playlist->playlist))
	{
		DumpErrorStack(_("Error updating playlist on device."));
		return false;
	}
	return true;
}

bool MTPDevice::SetPlaylistName(const Glib::RefPtr<playlist_t>& playlist, const std::string& name)
{
	std::string filename = name + ".pla";
	if (LIBMTP_Set_Playlist_Name(m_Device, playlist->playlist, filename.c_str()))
	{
		DumpErrorStack(_("Failed to change playlist name on device."));
		return false;
	}
	else
	{
		playlist->SetName(name);
		// Change the playlist name on the device
		UpdatePlaylist(playlist);
	}

	// Get the playlist name the device actually used (may be different
	// due to device naming restrictions)
	playlist->SetName(GetPlaylist(playlist->Id())->Name());

	return true;
}

Glib::RefPtr<playlist_t> MTPDevice::GetPlaylist(const uint32_t id)
{
	return playlist_t::create(LIBMTP_Get_Playlist(m_Device, id));
}

bool MTPDevice::CreateNewPlaylist(const Glib::RefPtr<playlist_t>& playlist)
{
	if (LIBMTP_Create_New_Playlist(m_Device, playlist->playlist))
	{
		DumpErrorStack(_("Failed to create playlist on device."));
		return false;
	}
	return true;
}

bool MTPDevice::CreateNewTrack(const Glib::ustring& filename,
		const Glib::RefPtr<track_t>& track)
{
	if (LIBMTP_Send_Track_From_File(m_Device, filename.c_str(), track->track,
				m_ProgressCallback, m_CallbackData))
	{
		DumpErrorStack(_("Failed to create track on device."));
		m_ProgressCallback(0, 1, m_CallbackData);
		return false;
	}
	m_ProgressCallback(0, 1, m_CallbackData);
	return true;
}

// Update metadata for track
bool MTPDevice::UpdateTrack(const Glib::RefPtr<track_t>& track)
{
	if (LIBMTP_Update_Track_Metadata(m_Device, track->track))
	{
		DumpErrorStack(_("Failed to update album on device."));
		return false;
	}
	return true;
}

bool MTPDevice::CreateNewAlbum(const Glib::RefPtr<album_t>& album)
{
	if (LIBMTP_Create_New_Album(m_Device, album->album))
	{
		DumpErrorStack(_("Failed to create album on device."));
		return false;
	}
	return true;
}

// Update metadata and tracks for album
bool MTPDevice::UpdateAlbum(const Glib::RefPtr<album_t>& album)
{
	if (LIBMTP_Update_Album(m_Device, album->album))
	{
		DumpErrorStack(_("Failed to update album on device."));
		return false;
	}
	return true;
}

bool MTPDevice::SetAlbumArt(const Glib::RefPtr<album_t>& album, const Glib::ustring& filename)
{
	albumart_t album_art;

	try
	{
		Glib::RefPtr<Gdk::Pixbuf> image = Gdk::Pixbuf::create_from_file(filename, 300, 300); 
		if (!image) return false;
		image->save_to_buffer(album_art.data->data, (gsize&)album_art.data->size, "jpeg");
	}
	catch (const Glib::Error& err)
	{
		std::cerr << Glib::ustring::compose(_("Error loading image file %1: "), filename) <<
			err.what() << std::endl;
		return false;
	}

	if (LIBMTP_Send_Representative_Sample(m_Device, album->Id(), album_art.data))
	{
		std::cerr << _("Error sending album art to device.") << std::endl;
		return false;
	}

	GetAlbumArt(album);

	return true;
}

// Delete an object from the device
bool MTPDevice::DeleteObject(const uint32_t id)
{
	if (!id) return false;

	if (LIBMTP_Delete_Object(m_Device, id))
	{
		DumpErrorStack(_("Failed to delete object from device."));
		return false;
	}
	return true;
}

// Return a list of playlists on the device sorted in lexicographical order
void MTPDevice::GetPlaylistList(std::vector< Glib::RefPtr<playlist_t> >& playlist_list)
{
	LIBMTP_playlist_t* playlists = LIBMTP_Get_Playlist_List(m_Device);

	// Put the playlists into the list
	for (LIBMTP_playlist_t *playlist = playlists; playlist != NULL; playlist = playlist->next)
		playlist_list.push_back(playlist_t::create(playlist));

	std::sort(playlist_list.begin(), playlist_list.end(), &PlaylistIsLess);
}

// Add references in the album to all of the tracks listed in the album.
static void GetTracksInAlbumById(album_pair_t& album,
		std::vector< std::pair<Glib::RefPtr<track_t>, bool> >& track_list)
{
	if (track_list.empty() || !album.first || !album.first->NumTracks())
		return;

	std::vector<uint32_t> album_tracks = album.first->Tracks();
	for (std::vector<uint32_t>::iterator album_track = album_tracks.begin();
			album_track != album_tracks.end(); ++album_track)
	{
		std::vector< std::pair<Glib::RefPtr<track_t>, bool> >::iterator track = track_list.begin();
		for (; track != track_list.end() &&
				*album_track != track->first->Id(); ++track);
		if (track != track_list.end())
		{
			album.second.push_back(track->first);
			track->first->art = album.first->art;
			track->second = true;
		}
		else
			std::cerr << _("Error:  Track ") << *album_track << _(" referenced in album ") <<
				album.first->Name() << _(" not found on device.") << std::endl;
	}
}

// Goes all the way through the track list and finds any tracks whose album
// name is the same as that of the album name and have not already been
// included in an album, and references them in the album.
static void GetTracksInAlbumByName(album_pair_t& album,
		std::vector< std::pair<Glib::RefPtr<track_t>, bool> >& track_list)
{
	if (track_list.empty())
		return;

	std::vector<uint32_t> album_tracks = album.first->Tracks();
	std::vector< std::pair<Glib::RefPtr<track_t>, bool> >::iterator track = track_list.begin();
	while (track != track_list.end())
	{
		if (!track->second &&
				album.first->Name().collate_key() == track->first->Album().collate_key())
		{
			album_tracks.push_back(track->first->Id());
			album.second.push_back(track->first);
			track->second = true;
		}
		else
			++track;
	}
	album.first->SetTracks(album_tracks);
}

// Get the list of albums on the device, and sort the tracks into the albums,
// then add fake albums for tracks that are not in an album on the device.
void MTPDevice::GetMusicLists(std::vector<album_pair_t>& album_list)
{
	LIBMTP_track_t* tracks =
		LIBMTP_Get_Tracklisting_With_Callback(m_Device, m_ProgressCallback, m_CallbackData);

	// Put the tracks into a list
	std::vector< std::pair<Glib::RefPtr<track_t>, bool> > track_list;
	for (LIBMTP_track_t *track = tracks; track != NULL; track = track->next)
		track_list.push_back(std::make_pair(track_t::create(track), false));

	LIBMTP_album_t* albums = LIBMTP_Get_Album_List(m_Device);

	// Put the albums and their tracks into the list
	LIBMTP_album_t *current = albums;
	int num_tracks = 0;
	while (current != NULL)
	{
		album_pair_t this_album(album_t::create(current),
				std::vector< Glib::RefPtr<track_t> >());
		GetAlbumArt(this_album.first);
		GetTracksInAlbumById(this_album, track_list);
		album_list.push_back(this_album);
		m_ProgressCallback(num_tracks += this_album.second.size(), track_list.size(), m_CallbackData);
		current = current->next;
	}

	// If there are any remaining tracks, add them into the list in an new album
	for (std::vector< std::pair<Glib::RefPtr<track_t>, bool> >::iterator track = track_list.begin();
			track != track_list.end(); ++track)
	{
		if (track->second) continue;

		album_pair_t this_album(album_t::create(),
				std::vector< Glib::RefPtr<track_t> >());
		this_album.first->SetName(track->first->RawAlbum());
		this_album.first->SetArtist(track->first->RawArtist());
		this_album.first->SetGenre(track->first->Genre());
		this_album.first->SetTracks(std::vector<uint32_t>(1, track->first->Id()));
		this_album.second.push_back(track->first);
		track->second = true;

		GetTracksInAlbumByName(this_album, track_list);

		album_list.push_back(this_album);
		m_ProgressCallback(num_tracks += this_album.second.size(), track_list.size(), m_CallbackData);
	}
}
